@@ -1,16 +1,15 @@
 #  You may distribute under the terms of either the GNU General Public License
 #  or the Artistic License (the same terms as Perl itself)
 #
-#  (C) Paul Evans, 2014-2015 -- leonerd@leonerd.org.uk
+#  (C) Paul Evans, 2014 -- leonerd@leonerd.org.uk
 
 package Test::Future;
 
-use v5.10;
 use strict;
 use warnings;
 use base qw( Test::Builder::Module );
 
-our $VERSION = '0.48_003';
+our $VERSION = '0.29';
 
 our @EXPORT = qw(
    no_pending_futures
@@ -46,11 +45,7 @@ code based on, or using L<Future> instances or subclasses.
 
 =cut
 
-=head2 no_pending_futures
-
-   no_pending_futures( \&code, $name )
-
-I<Since version 0.29.>
+=head2 no_pending_futures( \&code, $name )
 
 Runs the given block of code, while keeping track of every C<Future> instance
 constructed while doing so. After the code has returned, each of these
@@ -77,35 +72,11 @@ sub no_pending_futures(&@)
 
    my @futures;
 
-   no warnings 'redefine';
-
    my $new = Future->can( "new" );
+   no warnings 'redefine';
    local *Future::new = sub {
       my $f = $new->(@_);
       push @futures, $f;
-      $f->on_ready( sub {
-         my $f = shift;
-         for ( 0 .. $#futures ) {
-            refaddr( $futures[$_] ) == refaddr( $f ) or next;
-
-            splice @futures, $_, 1, ();
-            return;
-         }
-      });
-      return $f;
-   };
-
-   my $done = Future->can( "done" );
-   local *Future::done = sub {
-      my $f = $done->(@_);
-      pop @futures if !ref $_[0]; # class method
-      return $f;
-   };
-
-   my $fail = Future->can( "fail" );
-   local *Future::fail = sub {
-      my $f = $fail->(@_);
-      pop @futures if !ref $_[0]; # class method
       return $f;
    };
 
